/*
 * Support for initialising programs in new tasks and threads.
 *
 * Copyright (C) 2022, 2023 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/re/env.h>
#include <l4/re/l4aux.h>
#include <l4/sys/factory.h>
#include <l4/sys/task.h>
#include <l4/sys/thread.h>

#include <exec/stack.h>



/* UTCB properties.
   See: moe/server/src/loader.cc */

enum exec_utcb
{
  Default_max_threads = 16,
#ifdef ARCH_mips
  Utcb_area_start = 0x73000000,
#else
  Utcb_area_start = 0xb3000000,
#endif
};



/* Capability index definitions. */

enum exec_task_caps
{
  L4_EXEC_PAGER_CAP  = 0x10UL << L4_CAP_SHIFT,
  L4_EXEC_RM_CAP     = 0x11UL << L4_CAP_SHIFT,
  L4_EXEC_MA_CAP     = 0x12UL << L4_CAP_SHIFT,
  L4_EXEC_PARENT_CAP = 0x13UL << L4_CAP_SHIFT,
  L4_EXEC_KIP_CAP    = 0x14UL << L4_CAP_SHIFT,
};

/* The default first free capability index must follow those above. Any
   additional initial capabilities must elevate the index below. */

enum exec_task_cap_indexes
{
  L4_EXEC_FIRST_FREE_CAP_INDEX  = 0x15UL,
};



/* A process abstraction. */

class Process
{
protected:
  l4_cap_idx_t _task = L4_INVALID_CAP;
  unsigned int _thread_number;

  /* Common and thread environment details. */

  l4re_aux_t _aux;
  l4re_env_t _env;

  /* Task and thread initialisation. */

  long create_task(unsigned int threads);

  long create_thread(l4_cap_idx_t *thread);

public:
  explicit Process();

  void reset();

  l4_cap_idx_t allocate_cap();

  long configure_task(l4_cap_idx_t *task, l4_cap_idx_t *mapped_task,
                      unsigned int threads = 2);

  long set_pager(l4_cap_idx_t rm, l4_cap_idx_t *mapped_rm = NULL);

  long set_parent(l4_cap_idx_t parent, l4_cap_idx_t *mapped_parent = NULL);

  long map_capabilities(struct ipc_mapped_cap mapped_caps[],
                        bool to_count = true);

  long thread_start(l4_addr_t program_start, Stack &st, l4_cap_idx_t *thread,
                    l4_cap_idx_t *mapped_thread);
};

/* vim: tabstop=2 expandtab shiftwidth=2
*/
